#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_form.interfaces.form import IInnerTabForm
from pyams_security.interfaces.profile import IPublicProfile
from pyams_skin.interfaces.viewlet import IUserMenusViewletManager
from pyams_skin.layer import IPyAMSLayer
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.menu import MenuItem, MenuDivider
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm, InnerAdminEditForm
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Interface

from pyams_security import _


@viewlet_config(name='login', layer=IPyAMSLayer, manager=IUserMenusViewletManager, weight=1)
@template_config(template='templates/user-login.pt', layer=IPyAMSLayer)
class UserLoginMenuItem(MenuItem):
    """User login menu item"""


@viewlet_config(name='profile', layer=IPyAMSLayer, manager=IUserMenusViewletManager, weight=10)
class UserProfileMenuItem(MenuItem):
    """User profile menu item"""

    label = _("User profile")
    icon_class = 'fa-user'
    url = 'user-profile.html'
    modal_target = True


@pagelet_config(name='user-profile.html', layer=IPyAMSLayer)
class UserProfileEditForm(AdminDialogEditForm):
    """User profile edit form"""

    legend = _("Edit user profile")
    fields = field.Fields(Interface)

    ajax_handler = 'user-profile.json'
    edit_permission = None

    dialog_class = 'modal-large'

    @property
    def title(self):
        return self.request.principal.title

    def getContent(self):
        return self.request.root

    def check_mode(self):
        self.mode = INPUT_MODE


@view_config(name='user-profile.json', request_type=IPyAMSLayer, renderer='json', xhr=True)
class UserProfileAJAXEditForm(AJAXEditForm, UserProfileEditForm):
    """User profile edit form, JSON renderer"""


@adapter_config(name='public_profile',
                context=(Interface, IPyAMSLayer, UserProfileEditForm),
                provides=IInnerTabForm)
class PublicProfileTabForm(InnerAdminEditForm):
    """Public profile tab form"""

    tab_label = _("Public profile")
    legend = None

    fields = field.Fields(IPublicProfile)
    edit_permission = None

    label_css_class = 'control-label col-md-4'
    input_css_class = 'col-md-8'

    weight = 10

    def getContent(self):
        return IPublicProfile(self.request.principal)


@viewlet_config(name='logout-divider', layer=IPyAMSLayer, manager=IUserMenusViewletManager, weight=999)
class UserLogoutMenuDivider(MenuDivider):
    """User logout menu divider"""


@viewlet_config(name='logout-menu', layer=IPyAMSLayer, manager=IUserMenusViewletManager, weight=1000)
class UserLogoutMenuItem(MenuItem):
    """User logout menu item"""

    label = _("Logout")
    css_class = 'logout'
    icon_class = 'fa-sign-out'
    url = "logout.html"
